/*** Headers to Include ***/
#include <allegro.h>
#include <winalleg.h>
#include <stdio.h>
#include <stdlib.h>
#include "FSUIPC_User.h"

/*** Define Variables ***/
int x_pos = 0, y_pos = 0;
int size = 300;
int message_wait_time = 5;
int loop = 1, keypress;
int screen_colour_depth;
float scale_factor;
long knots;
float angle, knots_fp;
char buffer[100];
DWORD dwResult;
HDC hdc;
FILE *inifile;
BITMAP *scrat, *background, *pointer;
PALETTE palette;

/*** Define Subroutines ***/
int initialise_window( void );
int connect_to_sim( void );
int update_display( void );
int create_bitmaps( void );
int get_knots( void );
int load_ini_file( void );
int save_ini_file( void );
int act_on_keypress( void );
int close_program( void );
void exit_hook (void );


int main( void )
{
    load_ini_file();
    initialise_window();
    if( create_bitmaps() == -1 )
    { 
        textprintf_centre(screen, font, size / 2, size * 0.15, 65535,  "*** Error ***" ); 
        textprintf_centre(screen, font, size / 2, size * 0.40, 65535,  "No Graphics files" );
        textprintf_centre(screen, font, size / 2, size * 0.65, 65535,  "Press Key to Exit" );
        readkey(); 
        close_program(); 
        return( -1 ); 
    }
    update_display();
    connect_to_sim();
    while( loop == 1 )
	{
        get_knots();
        while( keypressed() )
        {
            act_on_keypress();
            update_display();
        }
        clear_keybuf();
        update_display();
	}
	save_ini_file();
	close_program();
    return (0);
}

END_OF_MAIN()

int initialise_window( void )
{
    hdc = GetDC(NULL);
    screen_colour_depth = GetDeviceCaps (hdc, BITSPIXEL );
    
    if (allegro_init() != 0) return 1; //Initialise Allegro
    install_keyboard();
    
    set_color_depth(screen_colour_depth);
   
    
    if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 800, 600, 0, 0) != 0) {
		if (set_gfx_mode(GFX_SAFE, 800, 600, 0, 0) != 0) {
			set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
			allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
			return 1;
		}
    }
    
	SetWindowPos(win_get_window(), 0, x_pos, y_pos, size+6, size+25, SWP_ASYNCWINDOWPOS);
	set_display_switch_mode(SWITCH_BACKGROUND);
    scale_factor = ((float)size / 419);
    
    set_window_close_hook(exit_hook);
    return( 0 );
}

int connect_to_sim( void )
{
    knots_fp = 35.0;
    update_display();
    while (!FSUIPC_Open(SIM_ANY, &dwResult) && loop == 1 ) 
    {
		set_window_title( "Airspeed <<< Not Connected >>>" );
		Sleep( 100 );
		while( keypressed() )
		{
		    act_on_keypress();
		    update_display();
		}
		clear_keybuf();
	}
	set_window_title( "Airspeed >>>   Connected   <<<" );
	
	if (FSUIPC_Write(0x3380, 29, "Airspeed Indicator Connected\0", &dwResult)) FSUIPC_Process(&dwResult);
	if (FSUIPC_Write(0x32FA, 2, &message_wait_time, &dwResult)) FSUIPC_Process(&dwResult);
	
	return( 0 );
}

int update_display( void )
{
    if( knots_fp < 35 ) knots_fp = 35;
    else if( knots_fp > 262 ) knots_fp = 262;
    angle = (6.06e-8*pow(knots_fp,4)) - (3.98e-5*pow(knots_fp,3)) + (7.05e-3*pow(knots_fp,2)) + (0.78*knots_fp) - 18.28;
    //set_color_conversion(COLORCONV_16_TO_32);
    stretch_blit( background, scrat, 0,0,419,419,0,0,size,size);
    pivot_scaled_sprite(scrat, pointer, size / 2, (size / 2), 17, 167, ftofix(angle) , ftofix(scale_factor));
    blit(scrat, screen, 0, 0, 0, 0, size, size);
    Sleep(10);
    return( 0 );
}

int create_bitmaps( void )
{
    scrat = create_bitmap(size, size);
    if( (background = load_bitmap( "background.bmp", palette )) == NULL ) return( -1 );
    if( (pointer = load_bitmap( "pointer.bmp", palette )) == NULL ) return( -1 );
    return( 0 );
}

int get_knots( void )
{
      if(!FSUIPC_Read(0x02BC, 4, &knots, &dwResult) || !FSUIPC_Process(&dwResult)) { FSUIPC_Close(); connect_to_sim(); }
      knots_fp = (float)knots / 128;
      return( 0 );
}
int load_ini_file( void )
{
    if( (inifile = fopen( "airspeed_indicator.ini", "r" )) != NULL )
	{
		if( !feof( inifile ))																											
		{																											
			fgets( buffer, 99, inifile );																											
			x_pos = atoi( buffer );																											
		}																											
		if( !feof( inifile ))																											
		{																											
			fgets( buffer, 99, inifile );																											
			y_pos = atoi( buffer );																											
		}																											
		if( !feof( inifile ))																											
		{																											
			fgets( buffer, 99, inifile );																											
			size = atoi( buffer );																											
		}																														
		fclose( inifile );
	}
	return( 0 );
}
int save_ini_file( void )
{
    RECT rect;
    if (GetWindowRect(win_get_window(), &rect))
	{
		if( (inifile = fopen( "airspeed_indicator.ini", "w" )) != NULL )
		{
			fprintf( inifile, "%li\n",rect.left);
			fprintf( inifile, "%li\n",rect.top);
			fprintf( inifile, "%d\n",size );
            fprintf( inifile, "s=%d\n",screen_colour_depth );
			fclose( inifile );
		}
	}
    return( 0 );
}
int act_on_keypress( void )
{
	RECT rect;
	if (GetWindowRect(win_get_window(), &rect))
	{
		x_pos = (int)rect.left;
		y_pos = (int)rect.top;
	}
	keypress = readkey();
	switch (keypress)
	{
		case 2409:         //i pressed ( increase size )
		if ( size < 600 ) size+=1;
		destroy_bitmap( scrat );
		scrat = create_bitmap(size, size);
        SetWindowPos(win_get_window(), 0, x_pos, y_pos, size+6, size+25, SWP_NOZORDER);
        scale_factor = ((float)size / 419);
		break;
		
		case 1124:         //d pressed ( decrease size )
		if( size > 142 ) size-=1;
		destroy_bitmap( scrat );
		scrat = create_bitmap(size, size);
        SetWindowPos(win_get_window(), 0, x_pos, y_pos, size+6, size+25, SWP_NOZORDER);
        scale_factor = ((float)size / 419);
		break;
	}
	return( 0 );
}
int close_program( void )
{
    FSUIPC_Close();
    allegro_exit();
    return( 0 );
}
void exit_hook( void )
{
    loop = 0;
}
